@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2020 Ian McInerney <ian.s.mcinerney@ieee.org>
  * Copyright (C) 2007-2014 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.TXT for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.TXT for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,18 +25,20 @@
 
 #include <kiid.h>
 
+#include <boost/random/mersenne_twister.hpp>
 #include <boost/uuid/uuid_generators.hpp>
 #include <boost/uuid/uuid_io.hpp>
-#include <boost/functional/hash.hpp>
 
 #if BOOST_VERSION >= 106700
 #include <boost/uuid/entropy_error.hpp>
 #endif
 
-#include <nlohmann/json.hpp>
+#include <json_common.h>
 
 #include <cctype>
 #include <mutex>
+#include <utility>
+#include <stdlib.h>
 
 #include <wx/log.h>
 
@@ -73,8 +75,6 @@ KIID& NilUuid()
 
 KIID::KIID()
 {
-    m_cached_timestamp = 0;
-
 #if BOOST_VERSION >= 106700
     try
     {
@@ -102,18 +102,16 @@ KIID::KIID()
 
 
 KIID::KIID( int null ) :
-        m_uuid( nilGenerator() ),
-        m_cached_timestamp( 0 )
+        m_uuid( nilGenerator() )
 {
     wxASSERT( null == 0 );
 }
 
 
 KIID::KIID( const std::string& aString ) :
-        m_uuid(),
-        m_cached_timestamp( 0 )
+        m_uuid()
 {
-    if( aString.length() == 8
+    if( !aString.empty() && aString.length() <= 8
         && std::all_of( aString.begin(), aString.end(),
                         []( unsigned char c )
                         {
@@ -123,22 +121,23 @@ KIID::KIID( const std::string& aString ) :
         // A legacy-timestamp-based UUID has only the last 4 octets filled in.
         // Convert them individually to avoid stepping in the little-endian/big-endian
         // doo-doo.
-        for( int i = 0; i < 4; ++i )
+        for( int i = 0; i < 4; i++ )
         {
-            std::string octet = aString.substr( i * 2, 2 );
+            int start = static_cast<int>( aString.length() ) - 8 + i * 2;
+            int end = start + 2;
+
+            start = std::max( 0, start );
+            int len = std::max( 0, end - start );
+
+            std::string octet = aString.substr( start, len );
             m_uuid.data[i + 12] = strtol( octet.data(), nullptr, 16 );
         }
-
-        m_cached_timestamp = strtol( aString.c_str(), nullptr, 16 );
     }
     else
     {
         try
         {
             m_uuid = stringGenerator( aString );
-
-            if( IsLegacyTimestamp() )
-                m_cached_timestamp = strtol( aString.substr( 28 ).c_str(), nullptr, 16 );
         }
         catch( ... )
         {
@@ -206,18 +205,10 @@ bool KIID::SniffTest( const wxString& aCandidate )
 
 KIID::KIID( timestamp_t aTimestamp )
 {
-    m_cached_timestamp = aTimestamp;
-
-    // A legacy-timestamp-based UUID has only the last 4 octets filled in.
-    // Convert them individually to avoid stepping in the little-endian/big-endian
-    // doo-doo.
-    wxString str = AsLegacyTimestampString();
-
-    for( int i = 0; i < 4; ++i )
-    {
-        wxString octet      = str.substr( i * 2, 2 );
-        m_uuid.data[i + 12] = strtol( octet.data(), nullptr, 16 );
-    }
+    m_uuid.data[12] = static_cast<uint8_t>( aTimestamp >> 24 );
+    m_uuid.data[13] = static_cast<uint8_t>( aTimestamp >> 16 );
+    m_uuid.data[14] = static_cast<uint8_t>( aTimestamp >> 8 );
+    m_uuid.data[15] = static_cast<uint8_t>( aTimestamp );
 }
 
 
@@ -229,28 +220,26 @@ bool KIID::IsLegacyTimestamp() const
 
 timestamp_t KIID::AsLegacyTimestamp() const
 {
-    return m_cached_timestamp;
+    timestamp_t ret = 0;
+
+    ret |= m_uuid.data[12] << 24;
+    ret |= m_uuid.data[13] << 16;
+    ret |= m_uuid.data[14] << 8;
+    ret |= m_uuid.data[15];
+
+    return ret;
 }
 
 
 size_t KIID::Hash() const
 {
-    size_t hash = 0;
-
-    // Note: this is NOT little-endian/big-endian safe, but as long as it's just used
-    // at runtime it won't matter.
-
-    for( int i = 0; i < 4; ++i )
-        boost::hash_combine( hash, reinterpret_cast<const uint32_t*>( m_uuid.data )[i] );
-
-    return hash;
+    return boost::uuids::hash_value( m_uuid );
 }
 
 
 void KIID::Clone( const KIID& aUUID )
 {
-    m_uuid             = aUUID.m_uuid;
-    m_cached_timestamp = aUUID.m_cached_timestamp;
+    m_uuid = aUUID.m_uuid;
 }
 
 
@@ -260,6 +249,12 @@ wxString KIID::AsString() const
 }
 
 
+std::string KIID::AsStdString() const
+{
+    return boost::uuids::to_string( m_uuid );
+}
+
+
 wxString KIID::AsLegacyTimestampString() const
 {
     return wxString::Format( "%8.8lX", (unsigned long) AsLegacyTimestamp() );
@@ -271,8 +266,7 @@ void KIID::ConvertTimestampToUuid()
     if( !IsLegacyTimestamp() )
         return;
 
-    m_cached_timestamp = 0;
-    m_uuid             = randomGenerator();
+    m_uuid = randomGenerator();
 }
 
 
@@ -324,8 +318,11 @@ bool KIID_PATH::MakeRelativeTo( const KIID_PATH& aPath )
 
     for( size_t i = 0; i < aPath.size(); ++i )
     {
-        if( copy.at( i ).AsString() != aPath.at( i ).AsString() )
+        if( copy.at( i ) != aPath.at( i ) )
+        {
+            *this = copy;
             return false; // this path is not contained within aPath
+        }
     }
 
     for( size_t i = aPath.size(); i < copy.size(); ++i )
