@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2023 Alex Shvartzkop <dudesuchamazing@gmail.com>
- * Copyright (C) 2023-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +37,7 @@
 #include <wx/stdstream.h>
 #include <wx/zipstrm.h>
 
-#include <nlohmann/json.hpp>
+#include <json_common.h>
 #include <core/map_helpers.h>
 #include <wx/wfstream.h>
 
@@ -130,7 +130,7 @@ int SCH_IO_EASYEDA::GetModifyHash() const
 
 
 LIB_SYMBOL* loadSymbol( const wxString& aLibraryPath, nlohmann::json aFileData,
-                        const wxString& aAliasName, const STRING_UTF8_MAP* aProperties )
+                        const wxString& aAliasName, const std::map<std::string, UTF8>* aProperties )
 {
     SCH_EASYEDA_PARSER      parser( nullptr, nullptr );
     std::map<wxString, int> namesCounter;
@@ -228,7 +228,7 @@ LIB_SYMBOL* loadSymbol( const wxString& aLibraryPath, nlohmann::json aFileData,
                             LIB_SYMBOL* ksymbol = parser.ParseSymbol( origin, paramMap, parts );
 
                             // Clear reference numbers
-                            LIB_FIELD& refField = ksymbol->GetReferenceField();
+                            SCH_FIELD& refField = ksymbol->GetReferenceField();
                             wxString   origRef = refField.GetText();
                             wxString   reference;
 
@@ -276,7 +276,7 @@ LIB_SYMBOL* loadSymbol( const wxString& aLibraryPath, nlohmann::json aFileData,
             LIB_SYMBOL* ksymbol = parser.ParseSymbol( origin, *c_para, topDoc.shape );
 
             // Clear reference numbers
-            LIB_FIELD& refField = ksymbol->GetReferenceField();
+            SCH_FIELD& refField = ksymbol->GetReferenceField();
             wxString   origRef = refField.GetText();
             wxString   reference;
 
@@ -305,7 +305,7 @@ LIB_SYMBOL* loadSymbol( const wxString& aLibraryPath, nlohmann::json aFileData,
 
 void SCH_IO_EASYEDA::EnumerateSymbolLib( wxArrayString&         aSymbolNameList,
                                          const wxString&        aLibraryPath,
-                                         const STRING_UTF8_MAP* aProperties )
+                                         const std::map<std::string, UTF8>* aProperties )
 {
     std::map<wxString, int> namesCounter;
 
@@ -431,7 +431,7 @@ void SCH_IO_EASYEDA::EnumerateSymbolLib( wxArrayString&         aSymbolNameList,
 
 void SCH_IO_EASYEDA::EnumerateSymbolLib( std::vector<LIB_SYMBOL*>& aSymbolList,
                                          const wxString&           aLibraryPath,
-                                         const STRING_UTF8_MAP*    aProperties )
+                                         const std::map<std::string, UTF8>*    aProperties )
 {
     wxFFileInputStream in( aLibraryPath );
     nlohmann::json     js;
@@ -473,7 +473,7 @@ void SCH_IO_EASYEDA::EnumerateSymbolLib( std::vector<LIB_SYMBOL*>& aSymbolList,
 
 LIB_SYMBOL* SCH_IO_EASYEDA::LoadSymbol( const wxString&        aLibraryPath,
                                         const wxString&        aAliasName,
-                                        const STRING_UTF8_MAP* aProperties )
+                                        const std::map<std::string, UTF8>* aProperties )
 {
     try
     {
@@ -611,7 +611,7 @@ static void LoadSchematic( SCHEMATIC* aSchematic, SCH_SHEET* aRootSheet, const w
 
 SCH_SHEET* SCH_IO_EASYEDA::LoadSchematicFile( const wxString& aFileName, SCHEMATIC* aSchematic,
                                               SCH_SHEET*             aAppendToMe,
-                                              const STRING_UTF8_MAP* aProperties )
+                                              const std::map<std::string, UTF8>* aProperties )
 {
     wxCHECK( !aFileName.IsEmpty() && aSchematic, nullptr );
 
