@@ -226,16 +226,17 @@ public:
      */
     SCH_ITEM* Duplicate( bool doClone = false ) const;
 
-    static wxString GetUnitDescription( int aUnit );
-    static wxString GetBodyStyleDescription( int aBodyStyle );
-
     virtual void SetUnit( int aUnit ) { m_unit = aUnit; }
     int GetUnit() const { return m_unit; }
-    void SetUnitProp( int aUnit );
+
+    virtual wxString GetUnitDisplayName( int aUnit, bool aLabel ) const;
+    virtual wxString GetBodyStyleDescription( int aBodyStyle, bool aLabel ) const;
 
     virtual void SetBodyStyle( int aBodyStyle ) { m_bodyStyle = aBodyStyle; }
     int  GetBodyStyle() const { return m_bodyStyle; }
-    void SetBodyStyleProp( int aBodyStyle );
+
+    virtual void SetBodyStyleProp( const wxString& aBodyStyle );
+    virtual wxString GetBodyStyleProp() const;
 
     void SetPrivate( bool aPrivate ) { m_private = aPrivate; }
     bool IsPrivate() const { return m_private; }
@@ -593,6 +594,8 @@ public:
 
     virtual void SetStroke( const STROKE_PARAMS& aStroke ) { wxCHECK( false, /* void */ ); }
 
+    void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
+
     /**
      * Print an item.
      *
@@ -688,8 +691,6 @@ protected:
         bool operator()( const SCH_ITEM* aFirst, const SCH_ITEM* aSecond ) const;
     };
 
-    void getSymbolEditorMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList );
-
     /**
      * Provide the draw object specific comparison called by the == and < operators.
      *
