@@ -43,32 +43,6 @@
 // small margin in internal units between the pin text and the pin line
 #define PIN_TEXT_MARGIN 4
 
-wxString SCH_PIN::GetCanonicalElectricalTypeName( ELECTRICAL_PINTYPE aType )
-{
-    // These strings are the canonical name of the electrictal type
-    // Not translated, no space in name, only ASCII chars.
-    // to use when the string name must be known and well defined
-    // must have same order than enum ELECTRICAL_PINTYPE (see sch_pin.h)
-    static const wxChar* msgPinElectricType[] =
-    {
-        wxT( "input" ),
-        wxT( "output" ),
-        wxT( "bidirectional" ),
-        wxT( "tri_state" ),
-        wxT( "passive" ),
-        wxT( "free" ),
-        wxT( "unspecified" ),
-        wxT( "power_in" ),
-        wxT( "power_out" ),
-        wxT( "open_collector" ),
-        wxT( "open_emitter" ),
-        wxT( "no_connect" )
-    };
-
-    return msgPinElectricType[static_cast<int>( aType )];
-}
-
-
 /// Utility for getting the size of the 'internal' pin decorators (as a radius)
 /// i.e. the clock symbols (falling clock is actually external but is of
 /// the same kind)
@@ -334,29 +308,25 @@ void SCH_PIN::SetType( ELECTRICAL_PINTYPE aType )
 
 wxString SCH_PIN::GetCanonicalElectricalTypeName() const
 {
-    if( m_type == ELECTRICAL_PINTYPE::PT_INHERIT )
-    {
-        if( !m_libPin )
-            return GetCanonicalElectricalTypeName( ELECTRICAL_PINTYPE::PT_UNSPECIFIED );
+    if( m_type != ELECTRICAL_PINTYPE::PT_INHERIT )
+        return ::GetCanonicalElectricalTypeName( m_type );
 
-        return m_libPin->GetCanonicalElectricalTypeName();
-    }
+    if( !m_libPin )
+        return ::GetCanonicalElectricalTypeName( ELECTRICAL_PINTYPE::PT_UNSPECIFIED );
 
-    return GetCanonicalElectricalTypeName( m_type );
+    return m_libPin->GetCanonicalElectricalTypeName();
 }
 
 
 wxString SCH_PIN::GetElectricalTypeName() const
 {
-    if( m_type == ELECTRICAL_PINTYPE::PT_INHERIT )
-    {
-        if( !m_libPin )
-            return ElectricalPinTypeGetText( ELECTRICAL_PINTYPE::PT_UNSPECIFIED );
+    if( m_type != ELECTRICAL_PINTYPE::PT_INHERIT )
+        return ElectricalPinTypeGetText( m_type );
 
-        return m_libPin->GetElectricalTypeName();
-    }
+    if( !m_libPin )
+        return ElectricalPinTypeGetText( ELECTRICAL_PINTYPE::PT_UNSPECIFIED );
 
-    return ElectricalPinTypeGetText( m_type );
+    return m_libPin->GetElectricalTypeName();
 }
 
 
@@ -413,6 +383,34 @@ void SCH_PIN::SetName( const wxString& aName )
 }
 
 
+void SCH_PIN::SetAlt( const wxString& aAlt )
+{
+    // Do not set the alternate pin definition to the default pin name.  This breaks the library
+    // symbol comparison for the ERC and the library diff tool.  It also incorrectly causes the
+    // schematic symbol pin alternate to be set.
+    if( aAlt.IsEmpty() || aAlt == GetBaseName() )
+    {
+        m_alt = wxEmptyString;
+        return;
+    }
+
+    if( !m_libPin )
+    {
+        wxFAIL_MSG( wxString::Format( wxS( "Pin '%s' has no corresponding lib_pin" ), m_number ) );
+        m_alt = wxEmptyString;
+        return;
+    }
+
+    if( !m_libPin->GetAlternates().contains( aAlt ) )
+    {
+        wxFAIL_MSG( wxString::Format( wxS( "Pin '%s' has no alterate '%s'" ), m_number, aAlt ) );
+        m_alt = wxEmptyString;
+        return;
+    }
+
+    m_alt = aAlt;
+}
+
 bool SCH_PIN::IsDangling() const
 {
     if( GetType() == ELECTRICAL_PINTYPE::PT_NC || GetType() == ELECTRICAL_PINTYPE::PT_NIC )
@@ -447,11 +445,25 @@ bool SCH_PIN::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) cons
     const SCH_SEARCH_DATA& schSearchData =
             dynamic_cast<const SCH_SEARCH_DATA&>( aSearchData );
 
-    if( !schSearchData.searchAllPins )
-        return false;
+    if( schSearchData.searchAllPins
+        && ( EDA_ITEM::Matches( GetName(), aSearchData )
+             || EDA_ITEM::Matches( GetNumber(), aSearchData ) ) )
+    {
+        return true;
+    }
 
-    return EDA_ITEM::Matches( GetName(), aSearchData )
-                || EDA_ITEM::Matches( GetNumber(), aSearchData );
+    SCH_CONNECTION* connection = nullptr;
+    SCH_SHEET_PATH* sheetPath = reinterpret_cast<SCH_SHEET_PATH*>( aAuxData );
+
+    if( schSearchData.searchNetNames && sheetPath && ( connection = Connection( sheetPath ) ) )
+    {
+        wxString netName = connection->GetNetName();
+
+        if( EDA_ITEM::Matches( netName, aSearchData ) )
+            return true;
+    }
+
+    return false;
 }
 
 
@@ -1430,10 +1442,10 @@ void SCH_PIN::ChangeLength( int aLength )
         offsetX = -1 * lengthChange;
         break;
     case PIN_ORIENTATION::PIN_UP:
-        offsetY = lengthChange;
+        offsetY = -1 * lengthChange;
         break;
     case PIN_ORIENTATION::PIN_DOWN:
-        offsetY = -1 * lengthChange;
+        offsetY = lengthChange;
         break;
     }
 
@@ -1552,28 +1564,9 @@ void SCH_PIN::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITE
     wxString msg;
     SYMBOL*  symbol = GetParentSymbol();
 
-    if( dynamic_cast<LIB_SYMBOL*>( symbol ) )
-    {
-        getSymbolEditorMsgPanelInfo( aFrame, aList );
-    }
-    else
-    {
-        aList.emplace_back( _( "Type" ), _( "Pin" ) );
+    aList.emplace_back( _( "Type" ), _( "Pin" ) );
 
-        if( symbol->GetUnitCount() )
-        {
-            msg = m_libPin ? GetUnitDescription( m_libPin->GetUnit() ) :
-                             wxString( "Undefined library pin." );
-            aList.emplace_back( _( "Unit" ), msg );
-        }
-
-        if( symbol->HasAlternateBodyStyle() )
-        {
-            msg = m_libPin ? GetBodyStyleDescription( m_libPin->GetBodyStyle() ) :
-                             wxString( "Undefined library pin." );
-            aList.emplace_back( _( "Body Style" ), msg );
-        }
-    }
+    SCH_ITEM::GetMsgPanelInfo( aFrame, aList );
 
     aList.emplace_back( _( "Name" ), UnescapeString( GetShownName() ) );
     aList.emplace_back( _( "Number" ), GetShownNumber() );
@@ -1786,11 +1779,7 @@ bool SCH_PIN::HasConnectivityChanges( const SCH_ITEM* aItem,
 
 bool SCH_PIN::ConnectionPropagatesTo( const EDA_ITEM* aItem ) const
 {
-    if( !m_libPin )
-        return false;
-
-    // Reciprocal checking is done in CONNECTION_GRAPH anyway
-    return m_libPin->GetType() != ELECTRICAL_PINTYPE::PT_NC;
+    return GetType() != ELECTRICAL_PINTYPE::PT_NC;
 }
 
 
@@ -2165,7 +2154,7 @@ static struct SCH_PIN_DESC
                 .SetWriteableFunc( isSymbolEditor );
 
         propMgr.AddProperty( new PROPERTY<SCH_PIN, int>( _HKI( "Length" ),
-                    &SCH_PIN::SetLength, &SCH_PIN::GetLength,
+                    &SCH_PIN::ChangeLength, &SCH_PIN::GetLength,
                     PROPERTY_DISPLAY::PT_SIZE ) )
                 .SetWriteableFunc( isSymbolEditor );
 
