@@ -137,12 +137,26 @@ bool SCH_SHAPE::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) co
 }
 
 
+bool SCH_SHAPE::IsEndPoint( const VECTOR2I& aPt ) const
+{
+    SHAPE_T shape = GetShape();
+
+    if( ( shape == SHAPE_T::ARC ) || ( shape == SHAPE_T::BEZIER ) )
+        return ( aPt == GetStart() ) || ( aPt == GetEnd() );
+
+    return false;
+}
+
+
 void SCH_SHAPE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_OPTS& aPlotOpts,
                       int aUnit, int aBodyStyle, const VECTOR2I& aOffset, bool aDimmed )
 {
     if( IsPrivate() )
         return;
 
+    // note: if aBodyStyle == -1 the outline shape is not plotted. Only the filled area
+    // is plotted (used to plot cells for SCH_TABLE items
+
     SCH_RENDER_SETTINGS* renderSettings = getRenderSettings( aPlotter );
     int                  pen_size = GetEffectivePenWidth( renderSettings );
 
@@ -206,7 +220,7 @@ void SCH_SHAPE::Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_OPTS&
         else
             fill = FILL_T::NO_FILL;
 
-        pen_size = GetEffectivePenWidth( renderSettings );
+        pen_size = aBodyStyle == -1 ? 0 : GetEffectivePenWidth( renderSettings );
     }
 
     if( bg == COLOR4D::UNSPECIFIED || !aPlotter->GetColorMode() )
@@ -509,10 +523,7 @@ void SCH_SHAPE::Print( const SCH_RENDER_SETTINGS* aSettings, int aUnit, int aBod
 
 void SCH_SHAPE::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList )
 {
-    if( m_layer == LAYER_DEVICE )
-        getSymbolEditorMsgPanelInfo( aFrame, aList );
-    else
-        SCH_ITEM::GetMsgPanelInfo( aFrame, aList );
+    SCH_ITEM::GetMsgPanelInfo( aFrame, aList );
 
     ShapeGetMsgPanelInfo( aFrame, aList );
 }
