@@ -766,7 +766,8 @@ SCH_SHEET_LIST::SCH_SHEET_LIST( SCH_SHEET* aSheet )
 
 void SCH_SHEET_LIST::BuildSheetList( SCH_SHEET* aSheet, bool aCheckIntegrity )
 {
-    wxCHECK_RET( aSheet != nullptr, wxT( "Cannot build sheet list from undefined sheet." ) );
+    if( !aSheet )
+        return;
 
     std::vector<SCH_SHEET*> badSheets;
 
@@ -816,6 +817,68 @@ void SCH_SHEET_LIST::BuildSheetList( SCH_SHEET* aSheet, bool aCheckIntegrity )
 }
 
 
+void SCH_SHEET_LIST::SortByHierarchicalPageNumbers( bool aUpdateVirtualPageNums )
+{
+    for( const SCH_SHEET_PATH& path : *this )
+        path.CachePageNumber();
+
+    std::sort( begin(), end(),
+        []( const SCH_SHEET_PATH& a, const SCH_SHEET_PATH& b ) -> bool
+        {
+            // Find the divergence point in the paths
+            size_t common_len = 0;
+            size_t min_len = std::min( a.size(), b.size() );
+
+            while( common_len < min_len && a.at( common_len )->m_Uuid == b.at( common_len )->m_Uuid )
+                common_len++;
+
+            // If one path is a prefix of the other, the shorter one comes first
+            // This ensures parents come before children
+            if( common_len == a.size() )
+                return true;  // a is a prefix of b - a is the parent
+            if( common_len == b.size() )
+                return false; // b is a prefix of a - b is the parent
+
+            // Paths diverge at common_len
+            // If they share the same parent, sort by page number
+            // This ensures siblings are sorted by page number
+            SCH_SHEET* sheet_a = a.at( common_len );
+            SCH_SHEET* sheet_b = b.at( common_len );
+
+            // Create partial paths to get to these sheets for page number comparison
+            KIID_PATH ancestor;
+            for( size_t i = 0; i < common_len; i++ )
+                ancestor.push_back( a.at( i )->m_Uuid );
+
+            // Compare page numbers - use the last sheet's page number
+            wxString page_a = sheet_a->getPageNumber( ancestor );
+            wxString page_b = sheet_b->getPageNumber( ancestor );
+
+            int retval = SCH_SHEET::ComparePageNum( page_a, page_b );
+
+            if( retval != 0 )
+                return retval < 0;
+
+            // If page numbers are the same, use virtual page numbers as a tie-breaker
+            if( a.GetVirtualPageNumber() < b.GetVirtualPageNumber() )
+                return true;
+            else if( a.GetVirtualPageNumber() > b.GetVirtualPageNumber() )
+                return false;
+
+            // Finally, use UUIDs for stable ordering when everything else is equal
+            return a.GetCurrentHash() < b.GetCurrentHash();
+        } );
+
+    if( aUpdateVirtualPageNums )
+    {
+        int virtualPageNum = 1;
+
+        for( SCH_SHEET_PATH& sheet : *this )
+            sheet.SetVirtualPageNumber( virtualPageNum++ );
+    }
+}
+
+
 void SCH_SHEET_LIST::SortByPageNumbers( bool aUpdateVirtualPageNums )
 {
     for( const SCH_SHEET_PATH& path : *this )
@@ -878,11 +941,11 @@ bool SCH_SHEET_LIST::PageNumberExists( const wxString& aPageNumber ) const
 void SCH_SHEET_LIST::TrimToPageNumbers( const std::vector<wxString>& aPageInclusions )
 {
     auto it = std::remove_if( begin(), end(),
-                              [&]( SCH_SHEET_PATH sheet )
+                              [&]( const SCH_SHEET_PATH& sheet )
                               {
-                                  return std::find( aPageInclusions.begin(), aPageInclusions.end(),
-                                                    sheet.GetPageNumber() )
-                                         == aPageInclusions.end();
+                                  return std::find( aPageInclusions.begin(),
+                                                    aPageInclusions.end(),
+                                                    sheet.GetPageNumber() ) == aPageInclusions.end();
                               } );
 
     erase( it, end() );
@@ -1007,6 +1070,9 @@ void SCH_SHEET_LIST::AnnotatePowerSymbols()
     {
         wxString curr_ref = references[ii].GetRef();
 
+        if( curr_ref.IsEmpty() )
+            continue;
+
         if( ref_list.find( curr_ref ) == ref_list.end() )
         {
             ref_list[curr_ref] = ii;
@@ -1018,7 +1084,7 @@ void SCH_SHEET_LIST::AnnotatePowerSymbols()
             continue;   // not annotated
 
         // Duplicate: clear annotation by removing the number ending the ref
-        while( curr_ref.Last() >= '0' && curr_ref.Last() <= '9' )
+        while( !curr_ref.IsEmpty() && curr_ref.Last() >= '0' && curr_ref.Last() <= '9' )
             curr_ref.RemoveLast();
 
         references[ii].SetRef( curr_ref );
@@ -1247,7 +1313,7 @@ void SCH_SHEET_LIST::UpdateSheetInstanceData( const std::vector<SCH_SHEET_INSTAN
         wxCHECK2( sheet && path.Last(), continue );
 
         auto it = std::find_if( aSheetInstances.begin(), aSheetInstances.end(),
-                                [ path ]( const SCH_SHEET_INSTANCE& r ) -> bool
+                                [&path]( const SCH_SHEET_INSTANCE& r ) -> bool
                                 {
                                     return path.Path() == r.m_Path;
                                 } );
