@@ -330,6 +330,8 @@ void SCH_SYMBOL::UpdatePins()
         ii->second.erase( it );
         pin->SetLibPin( libPin );
         pin->SetPosition( libPin->GetPosition() );
+        pin->SetUnit( libPin->GetUnit() );
+        pin->SetBodyStyle( libPin->GetBodyStyle() );
 
         unassignedSchPins.erase( pin );
 
@@ -362,6 +364,8 @@ void SCH_SYMBOL::UpdatePins()
         m_pinMap[ libPin ] = pin;
         pin->SetLibPin( libPin );
         pin->SetPosition( libPin->GetPosition() );
+        pin->SetUnit( libPin->GetUnit() );
+        pin->SetBodyStyle( libPin->GetBodyStyle() );
         pin->SetNumber( libPin->GetNumber() );
 
         auto iii = altPinMap.find( libPin->GetNumber() );
@@ -440,19 +444,27 @@ int SCH_SYMBOL::GetUnitCount() const
 }
 
 
-wxString SCH_SYMBOL::GetUnitDisplayName( int aUnit ) const
+wxString SCH_SYMBOL::GetUnitDisplayName( int aUnit, bool aLabel ) const
 {
-    wxCHECK( m_part, ( wxString::Format( _( "Unit %s" ), SubReference( aUnit ) ) ) );
-
-    return m_part->GetUnitDisplayName( aUnit );
+    if( m_part )
+        return m_part->GetUnitDisplayName( aUnit, aLabel );
+    else if( aLabel )
+        return wxString::Format( _( "Unit %s" ), SubReference( aUnit ) );
+    else
+        return SubReference( aUnit );
 }
 
 
-bool SCH_SYMBOL::HasUnitDisplayName( int aUnit ) const
+wxString SCH_SYMBOL::GetBodyStyleDescription( int aBodyStyle, bool aLabel ) const
 {
-    wxCHECK( m_part, false );
-
-    return m_part->HasUnitDisplayName( aUnit );
+    if( m_part )
+        return m_part->GetBodyStyleDescription( aBodyStyle, aLabel );
+    else if( aBodyStyle == BODY_STYLE::DEMORGAN )
+        return aLabel ? _( "Alternate" ) : wxString( _HKI( "Alternate" ) );
+    else if( aBodyStyle == BODY_STYLE::BASE )
+        return aLabel ? _( "Standard" ) : wxString( _HKI( "Standard" ) );
+    else
+        return wxT( "?" );
 }
 
 
@@ -550,8 +562,8 @@ void SCH_SYMBOL::Print( const SCH_RENDER_SETTINGS* aSettings, int aUnit, int aBo
 }
 
 
-bool SCH_SYMBOL::GetInstance( SCH_SYMBOL_INSTANCE& aInstance,
-                              const KIID_PATH& aSheetPath, bool aTestFromEnd ) const
+bool SCH_SYMBOL::GetInstance( SCH_SYMBOL_INSTANCE& aInstance, const KIID_PATH& aSheetPath,
+                              bool aTestFromEnd ) const
 {
     for( const SCH_SYMBOL_INSTANCE& instance : m_instanceReferences )
     {
@@ -1224,15 +1236,12 @@ std::vector<SCH_PIN*> SCH_SYMBOL::GetPins( const SCH_SHEET_PATH* aSheet ) const
     if( aSheet )
         unit = GetUnitSelection( aSheet );
 
-    for( const std::unique_ptr<SCH_PIN>& p : m_pins )
+    for( const std::unique_ptr<SCH_PIN>& pin : m_pins )
     {
-        if( unit && p->GetLibPin() && p->GetLibPin()->GetUnit()
-            && ( p->GetLibPin()->GetUnit() != unit ) )
-        {
+        if( unit && pin->GetUnit() && pin->GetUnit() != unit )
             continue;
-        }
 
-        pins.push_back( p.get() );
+        pins.push_back( pin.get() );
     }
 
     return pins;
@@ -2084,6 +2093,11 @@ BITMAPS SCH_SYMBOL::GetMenuImage() const
 
 EMBEDDED_FILES* SCH_SYMBOL::GetEmbeddedFiles()
 {
+    std::unique_ptr<LIB_SYMBOL>& libSymbolRef = GetLibSymbolRef();
+
+    if( !libSymbolRef )
+        return nullptr;
+
     return GetLibSymbolRef()->GetEmbeddedFiles();
 }
 
@@ -2525,15 +2539,10 @@ bool SCH_SYMBOL::doIsConnected( const VECTOR2I& aPosition ) const
 
         // Collect only pins attached to the current unit and convert.
         // others are not associated to this symbol instance
-        int pin_unit      = pin->GetLibPin() ? pin->GetLibPin()->GetUnit()
-                                             : GetUnit();
-        int pin_bodyStyle = pin->GetLibPin() ? pin->GetLibPin()->GetBodyStyle()
-                                             : GetBodyStyle();
-
-        if( pin_unit > 0 && pin_unit != GetUnit() )
+        if( pin->GetUnit() > 0 && pin->GetUnit() != GetUnit() )
             continue;
 
-        if( pin_bodyStyle > 0 && pin_bodyStyle != GetBodyStyle() )
+        if( pin->GetBodyStyle() > 0 && pin->GetBodyStyle() != GetBodyStyle() )
             continue;
 
         if( pin->GetLocalPosition() == new_pos )
@@ -2981,13 +2990,37 @@ static struct SCH_SYMBOL_DESC
                     return false;
                 };
 
-        propMgr.AddProperty( new PROPERTY<SCH_SYMBOL, wxString>( _HKI( "Unit" ),
+        propMgr.AddProperty( new PROPERTY<SCH_SYMBOL, int>( _HKI( "Unit" ),
                     &SCH_SYMBOL::SetUnitProp, &SCH_SYMBOL::GetUnitProp ) )
-                .SetAvailableFunc( multiUnit );
+                .SetAvailableFunc( multiUnit )
+                .SetChoicesFunc( []( INSPECTABLE* aItem )
+                                 {
+                                     wxPGChoices choices;
 
-        propMgr.AddProperty( new PROPERTY<SCH_SYMBOL, int>( _HKI( "Body Style" ),
+                                     if( SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( aItem ) )
+                                     {
+                                         for( int ii = 1; ii <= symbol->GetUnitCount(); ii++ )
+                                             choices.Add( symbol->GetUnitDisplayName( ii, false ), ii );
+                                     }
+
+                                     return choices;
+                                 } );
+
+        propMgr.AddProperty( new PROPERTY<SCH_SYMBOL, wxString>( _HKI( "Body Style" ),
                     &SCH_SYMBOL::SetBodyStyleProp, &SCH_SYMBOL::GetBodyStyleProp ) )
-                .SetAvailableFunc( multiBodyStyle );
+                .SetAvailableFunc( multiBodyStyle )
+                .SetChoicesFunc( []( INSPECTABLE* aItem )
+                                 {
+                                     wxPGChoices choices;
+
+                                     if( SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( aItem ) )
+                                     {
+                                         for( int ii : { BODY_STYLE::BASE, BODY_STYLE::DEMORGAN } )
+                                             choices.Add( symbol->GetBodyStyleDescription( ii, false ) );
+                                     }
+
+                                     return choices;
+                                 } );
 
         const wxString groupAttributes = _HKI( "Attributes" );
 
