@@ -913,6 +913,18 @@ const EMBEDDED_FILES* SCHEMATIC::GetEmbeddedFiles() const
 }
 
 
+void SCHEMATIC::RunOnNestedEmbeddedFiles( const std::function<void( EMBEDDED_FILES* )>& aFunction )
+{
+    SCH_SCREENS screens( Root() );
+
+    for( SCH_SCREEN* screen = screens.GetFirst(); screen; screen = screens.GetNext() )
+    {
+        for( auto& [name, libSym] : screen->GetLibSymbols() )
+            aFunction( libSym->GetEmbeddedFiles() );
+    }
+}
+
+
 std::set<KIFONT::OUTLINE_FONT*> SCHEMATIC::GetFonts() const
 {
     std::set<KIFONT::OUTLINE_FONT*> fonts;
@@ -1277,6 +1289,7 @@ void SCHEMATIC::RecalculateConnections( SCH_COMMIT* aCommit, SCH_CLEANUP_FLAGS a
                                         PICKED_ITEMS_LIST* aLastChangeList )
 {
     SCHEMATIC_SETTINGS& settings = Settings();
+    RefreshHierarchy();
     SCH_SHEET_LIST      list = Hierarchy();
     SCH_COMMIT          localCommit( aToolManager );
 
