@@ -22,12 +22,14 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
+#include <filename_resolver.h>
 #include <pgm_base.h>
 #include <string>
 #include <string_utils.h>
 #include <common.h>
 #include <functional>
 #include <sch_symbol.h>
+#include <schematic.h>
 
 // Include simulator headers after wxWidgets headers to avoid conflicts with Windows headers
 // (especially on msys2 + wxWidgets 3.0.x)
@@ -53,20 +55,25 @@ void SIM_LIB_MGR::Clear()
 }
 
 
-wxString SIM_LIB_MGR::ResolveLibraryPath( const wxString& aLibraryPath, const PROJECT* aProject,
-                                          REPORTER& aReporter )
+wxString SIM_LIB_MGR::ResolveLibraryPath( const wxString& aLibraryPath, REPORTER& aReporter )
 {
-    wxString expandedPath = ExpandEnvVarSubstitutions( aLibraryPath, aProject );
+    FILENAME_RESOLVER resolver;
 
-    // Convert it to UNIX format
-    expandedPath.Replace( '\\', '/' );
+    resolver.SetProject( m_project );
+
+    std::vector<const EMBEDDED_FILES*> embeddedFilesStack;
+
+    for( const EMBEDDED_FILES* embeddedFiles : m_embeddedFilesStack )
+        embeddedFilesStack.push_back( embeddedFiles );
+
+    wxString expandedPath = resolver.ResolvePath( aLibraryPath, wxEmptyString, embeddedFilesStack );
 
     wxFileName fn( expandedPath );
 
     if( fn.IsAbsolute() )
         return fn.GetFullPath();
 
-    wxFileName projectFn( aProject ? aProject->AbsolutePath( expandedPath ) : expandedPath );
+    wxFileName projectFn( m_project ? m_project->AbsolutePath( expandedPath ) : expandedPath );
 
     if( projectFn.Exists() )
         return projectFn.GetFullPath();
@@ -106,7 +113,7 @@ wxString SIM_LIB_MGR::ResolveEmbeddedLibraryPath( const wxString& aLibPath,
     {
         wxString relLib( aRelativeLib );
 
-        relLib = ResolveLibraryPath( relLib, m_project, aReporter );
+        relLib = ResolveLibraryPath( relLib, aReporter );
 
         wxFileName fn( relLib );
 
@@ -119,7 +126,7 @@ wxString SIM_LIB_MGR::ResolveEmbeddedLibraryPath( const wxString& aLibPath,
     if( !fn.Exists() )
         fullPath = aLibPath;
 
-    fullPath = ResolveLibraryPath( fullPath, m_project, aReporter );
+    fullPath = ResolveLibraryPath( fullPath, aReporter );
 
     return fullPath;
 }
@@ -127,7 +134,7 @@ wxString SIM_LIB_MGR::ResolveEmbeddedLibraryPath( const wxString& aLibPath,
 
 void SIM_LIB_MGR::SetLibrary( const wxString& aLibraryPath, REPORTER& aReporter )
 {
-    wxString path = ResolveLibraryPath( aLibraryPath, m_project, aReporter );
+    wxString path = ResolveLibraryPath( aLibraryPath, aReporter );
 
     if( aReporter.HasMessageOfSeverity( RPT_SEVERITY_UNDEFINED | RPT_SEVERITY_ERROR ) )
         return;
@@ -175,7 +182,7 @@ SIM_MODEL& SIM_LIB_MGR::CreateModel( const SIM_MODEL* aBaseModel,
 }
 
 SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const SCH_SHEET_PATH* aSheetPath, SCH_SYMBOL& aSymbol,
-                                             REPORTER& aReporter )
+                                             bool aResolve, int aDepth, REPORTER& aReporter )
 {
     // Note: currently this creates a resolved model (all Kicad variables references are resolved
     // before building the model).
@@ -197,7 +204,7 @@ SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const SCH_SHEET_PATH* aSheetPath, S
                  || field.GetName().StartsWith( wxS( "Sim." ) ) )
         {
             fields.emplace_back( VECTOR2I(), -1, &aSymbol, field.GetName() );
-            fields.back().SetText( field.GetShownText( aSheetPath, false ) );
+            fields.back().SetText( field.GetShownText( aSheetPath, false, aDepth ) );
         }
     }
 
@@ -221,7 +228,7 @@ SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const SCH_SHEET_PATH* aSheetPath, S
     bool     storeInValue = false;
 
     // Infer RLC and VI models if they aren't specified
-    if( SIM_MODEL::InferSimModel( aSymbol, &fields, true, SIM_VALUE_GRAMMAR::NOTATION::SI,
+    if( SIM_MODEL::InferSimModel( aSymbol, &fields, aResolve, aDepth, SIM_VALUE_GRAMMAR::NOTATION::SI,
                                   &deviceType, &modelType, &modelParams, &pinMap ) )
     {
         getOrCreateField( SIM_DEVICE_FIELD )->SetText( deviceType );
@@ -243,7 +250,7 @@ SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const SCH_SHEET_PATH* aSheetPath, S
                    return StrNumCmp( lhs->GetNumber(), rhs->GetNumber(), true ) < 0;
                } );
 
-    SIM_LIBRARY::MODEL model = CreateModel( fields, sourcePins, true, aReporter );
+    SIM_LIBRARY::MODEL model = CreateModel( fields, aResolve, aDepth, sourcePins, aReporter );
 
     model.model.SetIsStoredInValue( storeInValue );
 
@@ -251,12 +258,11 @@ SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const SCH_SHEET_PATH* aSheetPath, S
 }
 
 
-SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const std::vector<SCH_FIELD>& aFields,
-                                             const std::vector<SCH_PIN*>& aPins, bool aResolved,
-                                             REPORTER& aReporter )
+SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const std::vector<SCH_FIELD>& aFields, bool aResolve, int aDepth,
+                                             const std::vector<SCH_PIN*>& aPins, REPORTER& aReporter )
 {
-    std::string libraryPath = SIM_MODEL::GetFieldValue( &aFields, SIM_LIBRARY::LIBRARY_FIELD );
-    std::string baseModelName = SIM_MODEL::GetFieldValue( &aFields, SIM_LIBRARY::NAME_FIELD );
+    std::string libraryPath = SIM_MODEL::GetFieldValue( &aFields, SIM_LIBRARY::LIBRARY_FIELD, aResolve, aDepth );
+    std::string baseModelName = SIM_MODEL::GetFieldValue( &aFields, SIM_LIBRARY::NAME_FIELD, aResolve, aDepth );
 
     if( libraryPath != "" )
     {
@@ -264,7 +270,7 @@ SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const std::vector<SCH_FIELD>& aFiel
     }
     else
     {
-        m_models.push_back( SIM_MODEL::Create( aFields, aPins, aResolved, aReporter ) );
+        m_models.push_back( SIM_MODEL::Create( aFields, aPins, aResolve, aDepth, aReporter ) );
         return { baseModelName, *m_models.back() };
     }
 }
@@ -280,7 +286,7 @@ SIM_LIBRARY::MODEL SIM_LIB_MGR::CreateModel( const wxString& aLibraryPath,
     SIM_LIBRARY* library = nullptr;
     SIM_MODEL*   baseModel = nullptr;
     std::string  modelName;
-    wxString     path = ResolveLibraryPath( aLibraryPath, m_project, aReporter );
+    wxString     path = ResolveLibraryPath( aLibraryPath, aReporter );
 
     auto it = m_libraries.find( path );
 
