@@ -342,7 +342,7 @@ void DIALOG_DRC::OnRunDRCClick( wxCommandEvent& aEvent )
 
     m_runningResultsBook->ChangeSelection( 0 );   // Display the "Tests Running..." tab
     m_messages->Clear();
-    wxYield();                                    // Allow time slice to refresh Messages
+    wxSafeYield();                                // Allow time slice to refresh Messages
 
     m_running = true;
     m_sdbSizerCancel->SetLabel( _( "Cancel" ) );
@@ -362,7 +362,7 @@ void DIALOG_DRC::OnRunDRCClick( wxCommandEvent& aEvent )
         m_messages->Report( _( "Done.<br><br>" ) );
 
     Raise();
-    wxYield();                                    // Allow time slice to refresh Messages
+    wxSafeYield();                                // Allow time slice to refresh Messages
 
     m_running = false;
     m_sdbSizerCancel->SetLabel( _( "Close" ) );
@@ -1184,6 +1184,7 @@ void DIALOG_DRC::ExcludeMarker()
         return;
 
     RC_TREE_NODE* node = RC_TREE_MODEL::ToNode( m_markerDataView->GetCurrentItem() );
+
     PCB_MARKER*   marker = dynamic_cast<PCB_MARKER*>( node->m_RcItem->GetParent() );
 
     if( marker && marker->GetSeverity() != RPT_SEVERITY_EXCLUSION )
@@ -1276,6 +1277,7 @@ void DIALOG_DRC::OnDeleteAllClick( wxCommandEvent& aEvent )
     }
 
     deleteAllMarkers( s_includeExclusions );
+    m_drcRun = false;
 
     refreshEditor();
     updateDisplayedCounts();
