@@ -199,6 +199,11 @@ void PCB_TABLE::Rotate( const VECTOR2I& aRotCentre, const EDA_ANGLE& aAngle )
 
 void PCB_TABLE::Flip( const VECTOR2I& aCentre, FLIP_DIRECTION aFlipDirection )
 {
+    EDA_ANGLE originalAngle = m_cells[0]->GetTextAngle();
+
+    Rotate( aCentre, -originalAngle );
+    Normalize();
+
     for( PCB_TABLECELL* cell : m_cells )
         cell->Flip( aCentre, aFlipDirection );
 
@@ -213,8 +218,27 @@ void PCB_TABLE::Flip( const VECTOR2I& aCentre, FLIP_DIRECTION aFlipDirection )
         rowOffset += GetColCount();
     }
 
+    VECTOR2I currentPos = GetPosition();
+
+    int      firstWidth = m_colWidths.begin()->second;
+    VECTOR2I translationVector = VECTOR2I( firstWidth, 0 );
+
+    VECTOR2I position = currentPos - translationVector;
+    SetPosition( position );
+
+    std::map<int, int> newColWidths;
+    for( int col = 0; col < GetColCount(); ++col )
+    {
+        newColWidths[col] = m_colWidths[GetColCount() - 1 - col];
+    }
+
+    m_colWidths = std::move( newColWidths );
+
     SetLayer( GetBoard()->FlipLayer( GetLayer() ) );
     Normalize();
+
+    Rotate( aCentre, originalAngle );
+    Normalize();
 }
 
 
@@ -225,6 +249,17 @@ void PCB_TABLE::RunOnChildren( const std::function<void( BOARD_ITEM* )>& aFuncti
 }
 
 
+void PCB_TABLE::RunOnDescendants( const std::function<void( BOARD_ITEM* )>& aFunction, int aDepth ) const
+{
+    for( PCB_TABLECELL* cell : m_cells )
+    {
+        aFunction( cell );
+
+        cell->RunOnDescendants( aFunction, aDepth + 1 );
+    }
+}
+
+
 const BOX2I PCB_TABLE::GetBoundingBox() const
 {
     // Note: a table with no cells is not allowed
@@ -236,14 +271,15 @@ const BOX2I PCB_TABLE::GetBoundingBox() const
 }
 
 
-void PCB_TABLE::DrawBorders( const std::function<void( const VECTOR2I& aPt1,
-                                                       const VECTOR2I& aPt2,
+void PCB_TABLE::DrawBorders( const std::function<void( const VECTOR2I& aPt1, const VECTOR2I& aPt2,
                                                        const STROKE_PARAMS& aStroke )>& aCallback ) const
 {
-    std::vector<VECTOR2I> topLeft     = GetCell( 0, 0 )->GetCornersInSequence();
-    std::vector<VECTOR2I> bottomLeft  = GetCell( GetRowCount() - 1, 0 )->GetCornersInSequence();
-    std::vector<VECTOR2I> topRight    = GetCell( 0, GetColCount() - 1 )->GetCornersInSequence();
-    std::vector<VECTOR2I> bottomRight = GetCell( GetRowCount() - 1, GetColCount() - 1 )->GetCornersInSequence();
+    EDA_ANGLE             drawAngle = GetCell( 0, 0 )->GetDrawRotation();
+    std::vector<VECTOR2I> topLeft = GetCell( 0, 0 )->GetCornersInSequence( drawAngle );
+    std::vector<VECTOR2I> bottomLeft = GetCell( GetRowCount() - 1, 0 )->GetCornersInSequence( drawAngle );
+    std::vector<VECTOR2I> topRight = GetCell( 0, GetColCount() - 1 )->GetCornersInSequence( drawAngle );
+    std::vector<VECTOR2I> bottomRight =
+            GetCell( GetRowCount() - 1, GetColCount() - 1 )->GetCornersInSequence( drawAngle );
     STROKE_PARAMS         stroke;
 
     for( int col = 0; col < GetColCount() - 1; ++col )
@@ -260,7 +296,10 @@ void PCB_TABLE::DrawBorders( const std::function<void( const VECTOR2I& aPt1,
             if( cell->GetColSpan() == 0 )
                 continue;
 
-            std::vector<VECTOR2I> corners = cell->GetCornersInSequence();
+            if( col + cell->GetColSpan() == GetColCount() )
+                continue;
+
+            std::vector<VECTOR2I> corners = cell->GetCornersInSequence( drawAngle );
 
             if( corners.size() == 4 )
                 aCallback( corners[1], corners[2], stroke );
@@ -283,7 +322,10 @@ void PCB_TABLE::DrawBorders( const std::function<void( const VECTOR2I& aPt1,
             if( cell->GetRowSpan() == 0 )
                 continue;
 
-            std::vector<VECTOR2I> corners = cell->GetCornersInSequence();
+            if( row + cell->GetRowSpan() == GetRowCount() )
+                continue;
+
+            std::vector<VECTOR2I> corners = cell->GetCornersInSequence( drawAngle );
 
             if( corners.size() == 4 )
                 aCallback( corners[2], corners[3], stroke );
@@ -302,10 +344,12 @@ void PCB_TABLE::DrawBorders( const std::function<void( const VECTOR2I& aPt1,
 
 std::shared_ptr<SHAPE> PCB_TABLE::GetEffectiveShape( PCB_LAYER_ID aLayer, FLASHING aFlash ) const
 {
-    std::vector<VECTOR2I> topLeft     = GetCell( 0, 0 )->GetCornersInSequence();
-    std::vector<VECTOR2I> bottomLeft  = GetCell( GetRowCount() - 1, 0 )->GetCornersInSequence();
-    std::vector<VECTOR2I> topRight    = GetCell( 0, GetColCount() - 1 )->GetCornersInSequence();
-    std::vector<VECTOR2I> bottomRight = GetCell( GetRowCount() - 1, GetColCount() - 1 )->GetCornersInSequence();
+    EDA_ANGLE             drawAngle = GetCell( 0, 0 )->GetDrawRotation();
+    std::vector<VECTOR2I> topLeft = GetCell( 0, 0 )->GetCornersInSequence( drawAngle );
+    std::vector<VECTOR2I> bottomLeft = GetCell( GetRowCount() - 1, 0 )->GetCornersInSequence( drawAngle );
+    std::vector<VECTOR2I> topRight = GetCell( 0, GetColCount() - 1 )->GetCornersInSequence( drawAngle );
+    std::vector<VECTOR2I> bottomRight =
+            GetCell( GetRowCount() - 1, GetColCount() - 1 )->GetCornersInSequence( drawAngle );
 
     std::shared_ptr<SHAPE_COMPOUND> shape = std::make_shared<SHAPE_COMPOUND>();
 
@@ -546,6 +590,24 @@ double PCB_TABLE::Similarity( const BOARD_ITEM& aOther ) const
 }
 
 
+size_t PCB_TABLE::GetHash() const
+{
+    size_t hash = 0;
+
+    // The goal here is to generate an hash that would change the least for
+    // mundane things like just moving the table.
+    hash_combine( hash, m_colCount );
+    hash_combine( hash, m_rowHeights.size() );
+
+    for( PCB_TABLECELL* cell : m_cells )
+    {
+        hash_combine( hash, cell->GetText() );
+    }
+
+    return hash;
+}
+
+
 static struct PCB_TABLE_DESC
 {
     PCB_TABLE_DESC()
