@@ -71,6 +71,13 @@ EDA_ITEM* PCB_TRACK::Clone() const
 }
 
 
+void PCB_TRACK::CopyFrom( const BOARD_ITEM* aOther )
+{
+    wxCHECK( aOther && aOther->Type() == PCB_TRACE_T, /* void */ );
+    *this = *static_cast<const PCB_TRACK*>( aOther );
+}
+
+
 PCB_ARC::PCB_ARC( BOARD_ITEM* aParent, const SHAPE_ARC* aArc ) :
     PCB_TRACK( aParent, PCB_ARC_T )
 {
@@ -86,6 +93,13 @@ EDA_ITEM* PCB_ARC::Clone() const
 }
 
 
+void PCB_ARC::CopyFrom( const BOARD_ITEM* aOther )
+{
+    wxCHECK( aOther && aOther->Type() == PCB_ARC_T, /* void */ );
+    *this = *static_cast<const PCB_ARC*>( aOther );
+}
+
+
 PCB_VIA::PCB_VIA( BOARD_ITEM* aParent ) :
         PCB_TRACK( aParent, PCB_VIA_T ),
         m_padStack( this )
@@ -136,6 +150,13 @@ PCB_VIA& PCB_VIA::operator=( const PCB_VIA &aOther )
 }
 
 
+void PCB_VIA::CopyFrom( const BOARD_ITEM* aOther )
+{
+    wxCHECK( aOther && aOther->Type() == PCB_VIA_T, /* void */ );
+    *this = *static_cast<const PCB_VIA*>( aOther );
+}
+
+
 EDA_ITEM* PCB_VIA::Clone() const
 {
     return new PCB_VIA( *this );
@@ -350,8 +371,6 @@ double PCB_VIA::Similarity( const BOARD_ITEM& aOther ) const
 
 void PCB_VIA::SetWidth( int aWidth )
 {
-    // This is present because of the parent class.  It should never be actually called on a via.
-    wxASSERT_MSG( false, "Warning: PCB_VIA::SetWidth called without a layer argument" );
     m_padStack.SetSize( { aWidth, aWidth }, PADSTACK::ALL_LAYERS );
 }
 
@@ -389,8 +408,7 @@ void PCB_TRACK::Serialize( google::protobuf::Any &aContainer ) const
     track.set_layer( ToProtoEnum<PCB_LAYER_ID, kiapi::board::types::BoardLayer>( GetLayer() ) );
     track.set_locked( IsLocked() ? kiapi::common::types::LockedState::LS_LOCKED
                                  : kiapi::common::types::LockedState::LS_UNLOCKED );
-    track.mutable_net()->mutable_code()->set_value( GetNetCode() );
-    track.mutable_net()->set_name( GetNetname() );
+    PackNet( track.mutable_net() );
     // TODO m_hasSolderMask and m_solderMaskMargin
 
     aContainer.PackFrom( track );
@@ -409,7 +427,7 @@ bool PCB_TRACK::Deserialize( const google::protobuf::Any &aContainer )
     SetEnd( VECTOR2I( track.end().x_nm(), track.end().y_nm() ) );
     SetWidth( track.width().value_nm() );
     SetLayer( FromProtoEnum<PCB_LAYER_ID, kiapi::board::types::BoardLayer>( track.layer() ) );
-    SetNetCode( track.net().code().value() );
+    UnpackNet( track.net() );
     SetLocked( track.locked() == kiapi::common::types::LockedState::LS_LOCKED );
     // TODO m_hasSolderMask and m_solderMaskMargin
 
@@ -432,8 +450,7 @@ void PCB_ARC::Serialize( google::protobuf::Any &aContainer ) const
     arc.set_layer( ToProtoEnum<PCB_LAYER_ID, kiapi::board::types::BoardLayer>( GetLayer() ) );
     arc.set_locked( IsLocked() ? kiapi::common::types::LockedState::LS_LOCKED
                                : kiapi::common::types::LockedState::LS_UNLOCKED );
-    arc.mutable_net()->mutable_code()->set_value( GetNetCode() );
-    arc.mutable_net()->set_name( GetNetname() );
+    PackNet( arc.mutable_net() );
     // TODO m_hasSolderMask and m_solderMaskMargin
 
     aContainer.PackFrom( arc );
@@ -453,7 +470,7 @@ bool PCB_ARC::Deserialize( const google::protobuf::Any &aContainer )
     SetEnd( VECTOR2I( arc.end().x_nm(), arc.end().y_nm() ) );
     SetWidth( arc.width().value_nm() );
     SetLayer( FromProtoEnum<PCB_LAYER_ID, kiapi::board::types::BoardLayer>( arc.layer() ) );
-    SetNetCode( arc.net().code().value() );
+    UnpackNet( arc.net() );
     SetLocked( arc.locked() == kiapi::common::types::LockedState::LS_LOCKED );
     // TODO m_hasSolderMask and m_solderMaskMargin
 
@@ -482,8 +499,7 @@ void PCB_VIA::Serialize( google::protobuf::Any &aContainer ) const
     via.set_type( ToProtoEnum<VIATYPE, kiapi::board::types::ViaType>( GetViaType() ) );
     via.set_locked( IsLocked() ? kiapi::common::types::LockedState::LS_LOCKED
                                : kiapi::common::types::LockedState::LS_UNLOCKED );
-    via.mutable_net()->mutable_code()->set_value( GetNetCode() );
-    via.mutable_net()->set_name( GetNetname() );
+    PackNet( via.mutable_net() );
 
     aContainer.PackFrom( via );
 }
@@ -510,7 +526,7 @@ bool PCB_VIA::Deserialize( const google::protobuf::Any &aContainer )
     m_padStack.LayerSet().reset();
 
     SetViaType( FromProtoEnum<VIATYPE>( via.type() ) );
-    SetNetCode( via.net().code().value() );
+    UnpackNet( via.net() );
     SetLocked( via.locked() == kiapi::common::types::LockedState::LS_LOCKED );
 
     return true;
@@ -962,9 +978,8 @@ bool PCB_TRACK::IsOnLayer( PCB_LAYER_ID aLayer ) const
         return true;
     }
 
-    if( m_hasSolderMask
-            && ( ( aLayer == F_Mask && m_layer == F_Cu )
-                       || ( aLayer == B_Mask && m_layer == B_Cu ) ) )
+    if( m_hasSolderMask && (   ( aLayer == F_Mask && m_layer == F_Cu )
+                            || ( aLayer == B_Mask && m_layer == B_Cu ) ) )
     {
         return true;
     }
@@ -1139,7 +1154,6 @@ void PCB_VIA::SetLayerSet( const LSET& aLayerSet )
 
 void PCB_VIA::SetLayerPair( PCB_LAYER_ID aTopLayer, PCB_LAYER_ID aBottomLayer )
 {
-
     Padstack().Drill().start = aTopLayer;
     Padstack().Drill().end = aBottomLayer;
     SanitizeLayers();
@@ -1148,13 +1162,24 @@ void PCB_VIA::SetLayerPair( PCB_LAYER_ID aTopLayer, PCB_LAYER_ID aBottomLayer )
 
 void PCB_VIA::SetTopLayer( PCB_LAYER_ID aLayer )
 {
-    Padstack().Drill().start = aLayer;
+    // refuse invalid via
+    if( aLayer == Padstack().Drill().end )
+        return;
+
+    Padstack().Drill().start = aLayer;    
+    SanitizeLayers();
+
 }
 
 
 void PCB_VIA::SetBottomLayer( PCB_LAYER_ID aLayer )
 {
+    // refuse invalid via
+    if( aLayer == Padstack().Drill().start )
+        return;
+
     Padstack().Drill().end = aLayer;
+    SanitizeLayers();
 }
 
 
@@ -1207,15 +1232,10 @@ void PCB_VIA::SanitizeLayers()
 
 bool PCB_VIA::FlashLayer( LSET aLayers ) const
 {
-    for( size_t ii = 0; ii < aLayers.size(); ++ii )
+    for( PCB_LAYER_ID layer : aLayers )
     {
-        if( aLayers.test( ii ) )
-        {
-            PCB_LAYER_ID layer = PCB_LAYER_ID( ii );
-
-            if( FlashLayer( layer ) )
-                return true;
-        }
+        if( FlashLayer( layer ) )
+            return true;
     }
 
     return false;
@@ -1450,7 +1470,7 @@ std::vector<int> PCB_VIA::ViewGetLayers() const
     LSET cuMask = LSET::AllCuMask();
 
     if( const BOARD* board = GetBoard() )
-        cuMask = board->GetEnabledLayers();
+        cuMask &= board->GetEnabledLayers();
 
     for( PCB_LAYER_ID layer : layers )
     {
@@ -1480,18 +1500,12 @@ double PCB_VIA::ViewGetLOD( int aLayer, const KIGFX::VIEW* aView ) const
 {
     PCB_PAINTER*         painter = static_cast<PCB_PAINTER*>( aView->GetPainter() );
     PCB_RENDER_SETTINGS* renderSettings = painter->GetSettings();
-    LSET                 visible = LSET::AllLayersMask();
+    const BOARD*         board = GetBoard();
 
     // Meta control for hiding all vias
     if( !aView->IsLayerVisible( LAYER_VIAS ) )
         return LOD_HIDE;
 
-    // Handle board visibility
-    if( const BOARD* board = GetBoard() )
-        visible = board->GetVisibleLayers() & board->GetEnabledLayers();
-
-    int width = GetWidth( ToLAYER_ID( aLayer ) );
-
     // In high contrast mode don't show vias that don't cross the high-contrast layer
     if( renderSettings->GetHighContrast() )
     {
@@ -1514,21 +1528,37 @@ double PCB_VIA::ViewGetLOD( int aLayer, const KIGFX::VIEW* aView ) const
 
     if( IsHoleLayer( aLayer ) )
     {
+        LSET visible;
+
+        if( board )
+        {
+            visible = board->GetVisibleLayers();
+            visible &= board->GetEnabledLayers();
+        }
+        else
+        {
+            visible = LSET::AllLayersMask();
+        }
+
         if( m_viaType == VIATYPE::THROUGH )
         {
             // Show a through via's hole if any physical layer is shown
-            if( !( visible & LSET::PhysicalLayersMask() ).any() )
+            visible &= LSET::PhysicalLayersMask();
+
+            if( !visible.any() )
                 return LOD_HIDE;
         }
         else
         {
             // Show a blind or micro via's hole if it crosses a visible layer
-            if( !( visible & GetLayerSet() ).any() )
+            visible &= GetLayerSet();
+
+            if( !visible.any() )
                 return LOD_HIDE;
         }
 
         // The hole won't be visible anyway at this scale
-        return (double) pcbIUScale.mmToIU( 0.25 ) / GetDrillValue();
+        return lodScaleForThreshold( aView, GetDrillValue(), pcbIUScale.mmToIU( 0.25 ) );
     }
     else if( IsNetnameLayer( aLayer ) )
     {
@@ -1540,17 +1570,35 @@ double PCB_VIA::ViewGetLOD( int aLayer, const KIGFX::VIEW* aView ) const
         }
         else
         {
+            LSET visible;
+
+            if( board )
+            {
+                visible = board->GetVisibleLayers();
+                visible &= board->GetEnabledLayers();
+            }
+            else
+            {
+                visible = LSET::AllLayersMask();
+            }
+
             // Hide netnames unless pad is flashed to a visible layer
             if( !FlashLayer( visible ) )
                 return LOD_HIDE;
         }
 
+        int width = GetWidth( ToLAYER_ID( aLayer ) );
+
         // Netnames will be shown only if zoom is appropriate
         return lodScaleForThreshold( aView, width, pcbIUScale.mmToIU( 10 ) );
     }
 
     if( !IsCopperLayer( aLayer ) )
+    {
+        int width = GetWidth( ToLAYER_ID( aLayer ) );
+
         return lodScaleForThreshold( aView, width, pcbIUScale.mmToIU( 0.6 ) );
+    }
 
     return LOD_SHOW;
 }
@@ -1872,7 +1920,7 @@ VECTOR2I PCB_ARC::GetPosition() const
 double PCB_ARC::GetRadius() const
 {
     auto center = CalcArcCenter( m_Start, m_Mid , m_End );
-    return center.Distance( m_Start );
+    return std::min( center.Distance( m_Start ), (double) INT_MAX / 2.0 );
 }
 
 
@@ -1906,6 +1954,10 @@ EDA_ANGLE PCB_ARC::GetArcAngleEnd() const
 
 bool PCB_ARC::IsDegenerated( int aThreshold ) const
 {
+    // We have lots of code that will blow up if the radius overflows an int.
+    if( GetRadius() >= (double)INT_MAX/2.0 )
+        return true;
+
     // Too small arcs cannot be really handled: arc center (and arc radius)
     // cannot be safely computed if the distance between mid and end points
     // is too small (a few internal units)
@@ -2123,7 +2175,7 @@ static struct TRACK_VIA_DESC
         propMgr.AddProperty( new PROPERTY<PCB_VIA, int>( _HKI( "Hole" ),
             &PCB_VIA::SetDrill, &PCB_VIA::GetDrillValue, PROPERTY_DISPLAY::PT_SIZE ), groupVia );
         propMgr.AddProperty( new PROPERTY_ENUM<PCB_VIA, PCB_LAYER_ID>( _HKI( "Layer Top" ),
-            &PCB_VIA::SetLayer, &PCB_VIA::GetLayer ), groupVia );
+            &PCB_VIA::SetTopLayer, &PCB_VIA::GetLayer ), groupVia );
         propMgr.AddProperty( new PROPERTY_ENUM<PCB_VIA, PCB_LAYER_ID>( _HKI( "Layer Bottom" ),
             &PCB_VIA::SetBottomLayer, &PCB_VIA::BottomLayer ), groupVia );
         propMgr.AddProperty( new PROPERTY_ENUM<PCB_VIA, VIATYPE>( _HKI( "Via Type" ),
