@@ -154,25 +154,34 @@ public:
 
     const wxString& AsString() const override
     {
-        const_cast<PCBEXPR_NETCLASS_VALUE*>( this )->Set(
-                m_item->GetEffectiveNetClass()->GetName() );
+        const_cast<PCBEXPR_NETCLASS_VALUE*>( this )->Set( m_item->GetEffectiveNetClass()->GetName() );
         return LIBEVAL::VALUE::AsString();
     }
 
     bool EqualTo( LIBEVAL::CONTEXT* aCtx, const VALUE* b ) const override
     {
         if( const PCBEXPR_NETCLASS_VALUE* bValue = dynamic_cast<const PCBEXPR_NETCLASS_VALUE*>( b ) )
-        {
-            return *( m_item->GetEffectiveNetClass() )
-                   == *( bValue->m_item->GetEffectiveNetClass() );
-        }
+            return *( m_item->GetEffectiveNetClass() ) == *( bValue->m_item->GetEffectiveNetClass() );
 
         if( b->GetType() == LIBEVAL::VT_STRING )
         {
-            if( m_item->GetEffectiveNetClass()->ContainsNetclassWithName( b->AsString() ) )
-                return true;
+            // Test constituent net class names. The effective net class name (e.g. CLASS1,CLASS2,OTHER_CLASS) is
+            // tested in the fallthrough condition.
+            for( const NETCLASS* nc : m_item->GetEffectiveNetClass()->GetConstituentNetclasses() )
+            {
+                const wxString& ncName = nc->GetName();
 
-            return m_item->GetEffectiveNetClass()->GetName() == b->AsString();
+                if( b->StringIsWildcard() )
+                {
+                    if( WildCompareString( b->AsString(), ncName, false ) )
+                        return true;
+                }
+                else
+                {
+                    if( ncName.IsSameAs( b->AsString(), false ) )
+                        return true;
+                }
+            }
         }
 
         return LIBEVAL::VALUE::EqualTo( aCtx, b );
@@ -181,16 +190,37 @@ public:
     bool NotEqualTo( LIBEVAL::CONTEXT* aCtx, const LIBEVAL::VALUE* b ) const override
     {
         if( const PCBEXPR_NETCLASS_VALUE* bValue = dynamic_cast<const PCBEXPR_NETCLASS_VALUE*>( b ) )
-        {
-            return *( m_item->GetEffectiveNetClass() )
-                   != *( bValue->m_item->GetEffectiveNetClass() );
-        }
+            return *( m_item->GetEffectiveNetClass() ) != *( bValue->m_item->GetEffectiveNetClass() );
 
         if( b->GetType() == LIBEVAL::VT_STRING )
         {
-            const bool isInConstituents =
-                    m_item->GetEffectiveNetClass()->ContainsNetclassWithName( b->AsString() );
-            const bool isFullName = m_item->GetEffectiveNetClass()->GetName() == b->AsString();
+            // Test constituent net class names
+            bool isInConstituents = false;
+
+            for( const NETCLASS* nc : m_item->GetEffectiveNetClass()->GetConstituentNetclasses() )
+            {
+                const wxString& ncName = nc->GetName();
+
+                if( b->StringIsWildcard() )
+                {
+                    if( WildCompareString( b->AsString(), ncName, false ) )
+                    {
+                        isInConstituents = true;
+                        break;
+                    }
+                }
+                else
+                {
+                    if( ncName.IsSameAs( b->AsString(), false ) )
+                    {
+                        isInConstituents = true;
+                        break;
+                    }
+                }
+            }
+
+            // Test effective net class name
+            const bool isFullName = LIBEVAL::VALUE::EqualTo( aCtx, b );
 
             return !isInConstituents && !isFullName;
         }
@@ -207,7 +237,8 @@ class PCBEXPR_COMPONENT_CLASS_VALUE : public LIBEVAL::VALUE
 {
 public:
     PCBEXPR_COMPONENT_CLASS_VALUE( BOARD_ITEM* aItem ) :
-            LIBEVAL::VALUE( wxEmptyString ), m_item( dynamic_cast<FOOTPRINT*>( aItem ) )
+            LIBEVAL::VALUE( wxEmptyString ),
+            m_item( dynamic_cast<FOOTPRINT*>( aItem ) )
     {};
 
     const wxString& AsString() const override
@@ -223,8 +254,7 @@ public:
 
     bool EqualTo( LIBEVAL::CONTEXT* aCtx, const VALUE* b ) const override
     {
-        if( const PCBEXPR_COMPONENT_CLASS_VALUE* bValue =
-                    dynamic_cast<const PCBEXPR_COMPONENT_CLASS_VALUE*>( b ) )
+        if( const PCBEXPR_COMPONENT_CLASS_VALUE* bValue = dynamic_cast<const PCBEXPR_COMPONENT_CLASS_VALUE*>( b ) )
         {
             if( !m_item || !bValue->m_item )
                 return LIBEVAL::VALUE::EqualTo( aCtx, b );
@@ -239,10 +269,23 @@ public:
 
         if( b->GetType() == LIBEVAL::VT_STRING )
         {
-            if( m_item->GetComponentClass()->ContainsClassName( b->AsString() ) )
-                return true;
+            // Test constituent component class names. The effective component class name
+            // (e.g. CLASS1,CLASS2,OTHER_CLASS) is tested in the fallthrough condition.
+            for( const COMPONENT_CLASS* cc : m_item->GetComponentClass()->GetConstituentClasses() )
+            {
+                const wxString& ccName = cc->GetFullName();
 
-            return m_item->GetComponentClass()->GetFullName() == b->AsString();
+                if( b->StringIsWildcard() )
+                {
+                    if( WildCompareString( b->AsString(), ccName, false ) )
+                        return true;
+                }
+                else
+                {
+                    if( ccName.IsSameAs( b->AsString(), false ) )
+                        return true;
+                }
+            }
         }
 
         return LIBEVAL::VALUE::EqualTo( aCtx, b );
@@ -250,8 +293,7 @@ public:
 
     bool NotEqualTo( LIBEVAL::CONTEXT* aCtx, const LIBEVAL::VALUE* b ) const override
     {
-        if( const PCBEXPR_COMPONENT_CLASS_VALUE* bValue =
-                    dynamic_cast<const PCBEXPR_COMPONENT_CLASS_VALUE*>( b ) )
+        if( const PCBEXPR_COMPONENT_CLASS_VALUE* bValue = dynamic_cast<const PCBEXPR_COMPONENT_CLASS_VALUE*>( b ) )
         {
             if( !m_item || !bValue->m_item )
                 return LIBEVAL::VALUE::NotEqualTo( aCtx, b );
@@ -266,9 +308,33 @@ public:
 
         if( b->GetType() == LIBEVAL::VT_STRING )
         {
-            const bool isInConstituents =
-                    m_item->GetComponentClass()->ContainsClassName( b->AsString() );
-            const bool isFullName = m_item->GetComponentClass()->GetFullName() == b->AsString();
+            // Test constituent component class names
+            bool isInConstituents = false;
+
+            for( const COMPONENT_CLASS* cc : m_item->GetComponentClass()->GetConstituentClasses() )
+            {
+                const wxString& ccName = cc->GetFullName();
+
+                if( b->StringIsWildcard() )
+                {
+                    if( WildCompareString( b->AsString(), ccName, false ) )
+                    {
+                        isInConstituents = true;
+                        break;
+                    }
+                }
+                else
+                {
+                    if( ccName.IsSameAs( b->AsString(), false ) )
+                    {
+                        isInConstituents = true;
+                        break;
+                    }
+                }
+            }
+
+            // Test effective component class name
+            const bool isFullName = LIBEVAL::VALUE::EqualTo( aCtx, b );
 
             return !isInConstituents && !isFullName;
         }
@@ -347,7 +413,7 @@ LIBEVAL::VALUE* PCBEXPR_VAR_REF::GetValue( LIBEVAL::CONTEXT* aCtx )
         {
             if( m_isOptional )
             {
-                auto val = item->Get<std::optional<int>>( it->second );
+                std::optional<int> val = item->Get<std::optional<int>>( it->second );
 
                 if( val.has_value() )
                     return new LIBEVAL::VALUE( static_cast<double>( val.value() ) );
@@ -361,7 +427,7 @@ LIBEVAL::VALUE* PCBEXPR_VAR_REF::GetValue( LIBEVAL::CONTEXT* aCtx )
         {
             if( m_isOptional )
             {
-                auto val = item->Get<std::optional<double>>( it->second );
+                std::optional<double> val = item->Get<std::optional<double>>( it->second );
 
                 if( val.has_value() )
                     return new LIBEVAL::VALUE( val.value() );
@@ -464,7 +530,7 @@ LIBEVAL::FUNC_CALL_REF PCBEXPR_UCODE::CreateFuncCall( const wxString& aName )
 
 
 std::unique_ptr<LIBEVAL::VAR_REF> PCBEXPR_UCODE::CreateVarRef( const wxString& aVar,
-                                                                const wxString& aField )
+                                                               const wxString& aField )
 {
     PROPERTY_MANAGER& propMgr = PROPERTY_MANAGER::Instance();
     std::unique_ptr<PCBEXPR_VAR_REF> vref;
