@@ -23,6 +23,8 @@
 #define __PNS_KICAD_IFACE_H
 
 #include <unordered_set>
+#include <unordered_map>
+#include <vector>
 
 #include "pns_router.h"
 
@@ -37,6 +39,9 @@ class PCB_TOOL_BASE;
 class FOOTPRINT;
 class PAD;
 class EDA_TEXT;
+class LENGTH_DELAY_CALCULATION_ITEM;
+class BOARD_ITEM;
+class EDA_GROUP;
 
 namespace PNS
 {
@@ -95,7 +100,7 @@ public:
 
     BOARD* GetBoard() const { return m_board; }
 
-    virtual EDA_UNITS GetUnits() const { return EDA_UNITS::MILLIMETRES; };
+    virtual EDA_UNITS GetUnits() const { return EDA_UNITS::MM; };
 
     PNS::RULE_RESOLVER* GetRuleResolver() override;
     PNS::DEBUG_DECORATOR* GetDebugDecorator() override;
@@ -108,7 +113,7 @@ protected:
     std::unique_ptr<PNS::SEGMENT> syncTrack( PCB_TRACK* aTrack );
     std::unique_ptr<PNS::ARC>     syncArc( PCB_ARC* aArc );
     std::unique_ptr<PNS::VIA>     syncVia( PCB_VIA* aVia );
-    bool syncTextItem( PNS::NODE* aWorld, PCB_TEXT* aText, PCB_LAYER_ID aLayer );
+    bool syncTextItem( PNS::NODE* aWorld, BOARD_ITEM* aItem, PCB_LAYER_ID aLayer );
     bool syncGraphicalItem( PNS::NODE* aWorld, PCB_SHAPE* aItem );
     bool syncZone( PNS::NODE* aWorld, ZONE* aZone, SHAPE_POLY_SET* aBoardOutline );
     bool inheritTrackWidth( PNS::ITEM* aItem, int* aInheritedWidth );
@@ -163,6 +168,9 @@ protected:
     KIGFX::VIEW_GROUP*              m_previewItems;
     std::unordered_set<BOARD_ITEM*> m_hiddenItems;
 
+    std::unordered_map<BOARD_ITEM*, PCB_GROUP*>               m_itemGroups;
+    std::unordered_map<BOARD_ITEM*, std::vector<BOARD_ITEM*>> m_replacementMap;
+
     PCB_TOOL_BASE*                  m_tool;
     std::unique_ptr<BOARD_COMMIT>   m_commit;
     int                             m_commitFlags;
