@@ -75,8 +75,7 @@ ZONE* TEARDROP_MANAGER::createTeardrop( TEARDROP_VARIANT aTeardropVariant,
     teardrop->SetZoneName( aTeardropVariant == TD_TYPE_PADVIA ? MAGIC_TEARDROP_PADVIA_NAME
                                                               : MAGIC_TEARDROP_TRACK_NAME );
     teardrop->SetIslandRemovalMode( ISLAND_REMOVAL_MODE::NEVER );
-    teardrop->SetBorderDisplayStyle( ZONE_BORDER_DISPLAY_STYLE::DIAGONAL_FULL,
-                                     pcbIUScale.mmToIU( 0.1 ), true );
+    teardrop->SetBorderDisplayStyle( ZONE_BORDER_DISPLAY_STYLE::INVISIBLE_BORDER, 0, false );
 
     SHAPE_POLY_SET* outline = teardrop->Outline();
     outline->NewOutline();
@@ -96,6 +95,42 @@ ZONE* TEARDROP_MANAGER::createTeardrop( TEARDROP_VARIANT aTeardropVariant,
 }
 
 
+ZONE* TEARDROP_MANAGER::createTeardropMask( TEARDROP_VARIANT aTeardropVariant,
+                                            std::vector<VECTOR2I>& aPoints, PCB_TRACK* aTrack ) const
+{
+    ZONE* teardrop = new ZONE( m_board );
+
+    teardrop->SetTeardropAreaType( aTeardropVariant == TD_TYPE_PADVIA ? TEARDROP_TYPE::TD_VIAPAD
+                                                                      : TEARDROP_TYPE::TD_TRACKEND );
+    teardrop->SetLayer( aTrack->GetLayer() == F_Cu ? F_Mask : B_Mask );
+    teardrop->SetMinThickness( pcbIUScale.mmToIU( 0.0254 ) );  // The minimum zone thickness
+    teardrop->SetIsFilled( false );
+    teardrop->SetIslandRemovalMode( ISLAND_REMOVAL_MODE::NEVER );
+    teardrop->SetBorderDisplayStyle( ZONE_BORDER_DISPLAY_STYLE::INVISIBLE_BORDER, 0, false );
+
+    SHAPE_POLY_SET* outline = teardrop->Outline();
+    outline->NewOutline();
+
+    for( const VECTOR2I& pt: aPoints )
+        outline->Append( pt.x, pt.y );
+
+    if( int expansion = aTrack->GetSolderMaskExpansion() )
+    {
+        teardrop->SetMinThickness( std::max( teardrop->GetMinThickness(), expansion ) );
+
+        outline->Inflate( expansion, CORNER_STRATEGY::CHAMFER_ALL_CORNERS,
+                          m_board->GetDesignSettings().m_MaxError );
+    }
+
+    // Until we know better (ie: pay for a potentially very expensive zone refill), the teardrop
+    // fill is the same as its outline.
+    teardrop->SetFilledPolysList( teardrop->GetLayer(), *teardrop->Outline() );
+    teardrop->SetIsFilled( true );
+
+    return teardrop;
+}
+
+
 void TEARDROP_MANAGER::RemoveTeardrops( BOARD_COMMIT& aCommit,
                                         const std::vector<BOARD_ITEM*>* dirtyPadsAndVias,
                                         const std::set<PCB_TRACK*>* dirtyTracks )
@@ -201,8 +236,11 @@ void TEARDROP_MANAGER::UpdateTeardrops( BOARD_COMMIT& aCommit,
                 continue;
             }
 
-            if( pad->HitTest( track->GetStart() ) && pad->HitTest( track->GetEnd() ) )
-                // The track is entirely inside the pad; cannot create a teardrop
+            bool startHitsPad = pad->HitTest( track->GetStart(), 0, track->GetLayer() );
+            bool endHitsPad = pad->HitTest( track->GetEnd(), 0, track->GetLayer() );
+
+            // if the track is entirely inside the pad, cannot create a teardrop
+            if( startHitsPad && endHitsPad )
                 continue;
 
             // Skip case where pad and the track are within a copper zone with the same net
@@ -219,6 +257,13 @@ void TEARDROP_MANAGER::UpdateTeardrops( BOARD_COMMIT& aCommit,
                 m_createdTdList.push_back( new_teardrop );
 
                 aCommit.Added( new_teardrop );
+
+                if( track->HasSolderMask() && ( track->GetLayer() == F_Cu || track->GetLayer() == B_Cu ) )
+                {
+                    ZONE* new_teardrop_mask = createTeardropMask( TD_TYPE_PADVIA, points, track );
+                    m_board->Add( new_teardrop_mask, ADD_MODE::BULK_INSERT );
+                    aCommit.Added( new_teardrop_mask );
+                }
             }
         }
 
@@ -255,6 +300,13 @@ void TEARDROP_MANAGER::UpdateTeardrops( BOARD_COMMIT& aCommit,
                 m_createdTdList.push_back( new_teardrop );
 
                 aCommit.Added( new_teardrop );
+
+                if( track->HasSolderMask() && ( track->GetLayer() == F_Cu || track->GetLayer() == B_Cu ) )
+                {
+                    ZONE* new_teardrop_mask = createTeardropMask( TD_TYPE_PADVIA, points, track );
+                    m_board->Add( new_teardrop_mask, ADD_MODE::BULK_INSERT );
+                    aCommit.Added( new_teardrop_mask );
+                }
             }
         }
     }
@@ -430,6 +482,13 @@ void TEARDROP_MANAGER::AddTeardropsOnTracks( BOARD_COMMIT& aCommit,
                     m_createdTdList.push_back( new_teardrop );
 
                     aCommit.Added( new_teardrop );
+
+                    if( track->HasSolderMask() && ( track->GetLayer() == F_Cu || track->GetLayer() == B_Cu ) )
+                    {
+                        ZONE* new_teardrop_mask = createTeardropMask( TD_TYPE_TRACKEND, points, track );
+                        m_board->Add( new_teardrop_mask, ADD_MODE::BULK_INSERT );
+                        aCommit.Added( new_teardrop_mask );
+                    }
                 }
             }
         }
