@@ -44,17 +44,18 @@ PCB_PICKER_TOOL::PCB_PICKER_TOOL() :
 
 bool PCB_PICKER_TOOL::Init()
 {
-    PCB_BASE_FRAME*    frame = getEditFrame<PCB_BASE_FRAME>();
-    MAGNETIC_SETTINGS  magneticSettings;
-    CONDITIONAL_MENU&  menu = m_menu->GetMenu();
+    CONDITIONAL_MENU& menu = m_menu->GetMenu();
 
-    if( frame )
-        magneticSettings = *frame->GetMagneticItemsSettings();
-
-    const auto snapIsSetToAllLayers =
-            [&]( const SELECTION& aSel )
+    auto snapIsSetToAllLayers =
+            [this]( const SELECTION& aSel )
             {
-                return magneticSettings.allLayers;
+                if( PCB_BASE_FRAME* frame = getEditFrame<PCB_BASE_FRAME>() )
+                {
+                    if( frame->GetMagneticItemsSettings() )
+                        return frame->GetMagneticItemsSettings()->allLayers;
+                }
+
+                return false;
             };
 
     // "Cancel" goes at the top of the context menu when a tool is active
@@ -67,7 +68,7 @@ bool PCB_PICKER_TOOL::Init()
 
     menu.AddSeparator( 1 );
 
-    if( frame )
+    if( PCB_BASE_FRAME* frame = getEditFrame<PCB_BASE_FRAME>() )
         frame->AddStandardSubMenus( *m_menu.get() );
 
     return true;
@@ -258,11 +259,12 @@ int PCB_PICKER_TOOL::SelectPointInteractively( const TOOL_EVENT& aEvent )
 
     statusPopup.SetText( wxGetTranslation( params.m_Prompt ) );
 
-    const auto sendPoint = [&]( const std::optional<VECTOR2I>& aPoint )
-    {
-        statusPopup.Hide();
-        params.m_Receiver->UpdatePickedPoint( aPoint );
-    };
+    const auto sendPoint =
+            [&]( const std::optional<VECTOR2I>& aPoint )
+            {
+                statusPopup.Hide();
+                params.m_Receiver->UpdatePickedPoint( aPoint );
+            };
 
     SetSnapping( true );
     SetCursor( KICURSOR::PLACE );
@@ -304,11 +306,13 @@ int PCB_PICKER_TOOL::SelectPointInteractively( const TOOL_EVENT& aEvent )
     // Drop into the main event loop
     Main( aEvent );
 
+    ClearHandlers();
     canvas()->SetStatusPopup( nullptr );
     frame()->PopTool( aEvent );
     return 0;
 }
 
+
 int PCB_PICKER_TOOL::SelectItemInteractively( const TOOL_EVENT& aEvent )
 {
     INTERACTIVE_PARAMS params = aEvent.Parameter<INTERACTIVE_PARAMS>();
@@ -323,11 +327,12 @@ int PCB_PICKER_TOOL::SelectItemInteractively( const TOOL_EVENT& aEvent )
 
     statusPopup.SetText( wxGetTranslation( params.m_Prompt ) );
 
-    const auto sendItem = [&]( const EDA_ITEM* aItem )
-    {
-        statusPopup.Hide();
-        params.m_Receiver->UpdatePickedItem( aItem );
-    };
+    const auto sendItem =
+            [&]( const EDA_ITEM* aItem )
+            {
+                statusPopup.Hide();
+                params.m_Receiver->UpdatePickedItem( aItem );
+            };
 
     SetCursor( KICURSOR::BULLSEYE );
     SetSnapping( false );
@@ -338,8 +343,7 @@ int PCB_PICKER_TOOL::SelectItemInteractively( const TOOL_EVENT& aEvent )
             {
                 m_toolMgr->RunAction( PCB_ACTIONS::selectionClear );
                 const PCB_SELECTION& sel = selectionTool->RequestSelection(
-                        []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector,
-                            PCB_SELECTION_TOOL* sTool )
+                        []( const VECTOR2I& aPt, GENERAL_COLLECTOR& aCollector, PCB_SELECTION_TOOL* sTool )
                         {
                         } );
 
@@ -381,6 +385,7 @@ int PCB_PICKER_TOOL::SelectItemInteractively( const TOOL_EVENT& aEvent )
     // Drop into the main event loop
     Main( aEvent );
 
+    ClearHandlers();
     canvas()->SetStatusPopup( nullptr );
     frame()->PopTool( aEvent );
     return 0;
