@@ -100,6 +100,10 @@
 // position) so that resolution will work as expected for translated addresses.
 // This is transparent to the rest of the toolchain.
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>  // Must come first
+#endif
+
 #include "common/windows/omap.h"
 
 #include <atlbase.h>
